%
% 'Cauchy7.m'
%
function [val] = Cauchy7(u,v,M,N,h,U)
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% Cauchy7(u,v,M,N,h,U) computes a vector 
% of the Cauchy transformed sinc functions 
% c_j defined in Sec. 1.5 -- see 
% Eq. (1.5.78) -- for performing the 
% Cauchy transform over the arc 
%
% {z in C : z = e^(it), u < t < v}
%
% of the complex plane.   
%
% A FORTRAN variant of this method was used 
% previously in a program, used for [SSc] to 
% compute conformal maps.  
%  
% The routine calls both Sincpoints7 and 
% weights7, as well as the transformation 
% phi7 of Sec 1.5.3.  Here u can be a vector 
% of complex function values.
% 
m = M+N+1;
n = length(U);
z = Sincpoints7(u,v,M,N,h);
w = weights7(u,v,M,N,h);
p = 2*pi*i;
a = exp(i*u);
b = exp(i*v);
%
for j = 1:n
    UU(j) = phi7(u,v,U(j));
end
%
V = pi/h*UU; 
%
% First compute the n x m `t' matrix
%
for j=1:n
    for k=1:m
        kk = k - M - 1;
        ww = kk*pi;
        narg = V(j) - ww;
        darg = p*(U(j)-z(k));
        if abs(darg) > eps
           val(j,k) = w(k)*(exp(i*narg) - 1)/darg;
        else
           val(j,k) = 0;
        end
    end
end
%
% Next, compute the left end Cauchy basis:
%
for j=1:n
    val1(j) = -1/p + (b - u(j))/(b - a)*(1/2 - U(j)/p);
    for k=2:m;
        val1(j) = val1(j) - (b-z(k))/(b-a)*val(j,k);
    end;
%
% Finally, compute the right end Cauchy basis --
%
    valm(j) = 1/p + (u(j) - a)/(b - a)*(1/2 - U(j)/p);
    for k=1:m-1;
        valm(j) = valm(j) - (z(k) - a)/(b - a)*val(j,k);
    end;
%
    val(j,1) = val1(j);
    val(j,m) = valm(j);
end
%


       
    
