
%
% `Quad4.m'
%
function int = Quad4(Eps)
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% Essentially Trapezoidal - Midordinate
% iterative quadrature over (0,infinity)
% of a function F such that 
% f(x) =  tanh(x)F(x) satisfies 
% the conditions of Transformation #4 
% in Sec. 1.5.3.
%
format long e
%
Esq = sqrt(Eps);
h = 2;
n = 0;
t = 0;
er1 = 1;
er2 = 1;
er = er1+er2;
%
% This routine is written for a function FF4 
% See `FF4.m'.
%
while(er > Eps)
   z = psi4(-n*h);
   w = psi4_1(-n*h); 
   r = w*FF4(z);
   t = t+r;
   er3 = abs(r);
   er = er1+er2+er3;
   n = n+1;
   er1 = er2;
   er2 = er3;
end
%
M = n;
%
n = 1;
er1 = 1;
er2 = 1;
er = er1+er2;
while(er > Eps)
   z = psi4(n*h);
   w = psi4_1(n*h); 
   r = w*FF4(z);
   t = t+r;
   er3 = abs(r);
   er = er1+er2+er3;
   n = n+1;
   er1 = er2;
   er2 = er3;
end
%
N = n;
t = h*t;
%
r3 = 1;
while(r3 > Esq)
  em = 0;
  mm = M + N;
%
  for j = 1:mm
      xj = (j - M - 1/2)*h;
      z = psi4(xj);
      w = psi4_1(xj);
      em = em + w*FF4(z);
  end
%
  em = h*em;
%
  rr3 = abs(t-em);
  t = (t+em)/2;
% points = 2*(M+N) + 1;
  M = 2*M;
  N = 2*N;
h = h/2;
  r3 = rr3;
end
  %
  int = t;
%
% Output
%
% sprintf('%s','# of evaluation points = ',points)
% sprintf('%s %20.16e','Integral Value to Within Eps = ',t)

