%
% `Quad7.m'
%
function int = Quad7(u,v,Eps)
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% Essentially Trapezoidal - Midordinate
% iterative quadrature over the arc 
% z = e^{it}, u < t < v,
% to within an error of Eps, 
% of a function F with 
% f(x) = (x-e^{iu})(x-e^{iv})F(x) 
% satisfying the conditions of 
% Transformation #7 in Sec. 1.5.3.
%
format long e
%
Esq = sqrt(Eps);
h = 2;
n = 0;
t = 0;
er1 = 1;
er2 = 1;
er = er1+er2;
%
% This routine is written for a function FF7
% See `FF7.m'.
%
while(er > Eps)
   z = psi7(u,v,-n*h);
   w = psi7_1(u,v,-n*h); 
   r = w*FF7(z);
   t = t+r;
   er3 = abs(r);
   er = er1+er2+er3;
   n = n+1;
   er1 = er2;
   er2 = er3;
end
%
M = n;
%
n = 1;
er1 = 1;
er2 = 1;
er = er1+er2;
while(er > Eps)
   z = psi7(u,v,n*h);
   w = psi7_1(u,v,n*h); 
   r = w*FF7(z);
   t = t+r;
   er3 = abs(r);
   er = er1+er2+er3;
   n = n+1;
   er1 = er2;
   er2 = er3;
end
%
N = n;
t = h*t;
%
r3 = 1;
while(r3 > Esq)
  em = 0;
  mm = M + N;
%
  for j = 1:mm
      xj = (j - M - 1/2)*h;
      z = psi7(u,v,xj);
      w = psi7_1(u,v,xj);
      em = em + w*FF7(z);
  end
%
  em = h*em;
%
  rr3 = abs(t-em);
  t = (t+em)/2;
% points = 2*(M+N) + 1;
  M = 2*M;
  N = 2*N;
h = h/2;
  r3 = rr3;
end
  %
  int = t;
%
% Output
%
% sprintf('%s','# of evaluation points = ',points)
% sprintf('%s %20.16e','Integral Value to Within Eps = ',t)

