function [val] = basis(u,M,N,h)
%
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% basis(u,M,N,h) computes a vector of 
% the sinc functions omega_j -- see  
% Def. 1.5.12 and Thm. 1.5.13 of 
% this manual. This routine is called
% by each program basis*.m .  It is, 
% in fact, just the same as basis1.m ,
% and it requires proper definition of 
% u before it is called.
% 
  m = M+N+1;
  v = pi/h*u;
%
% First compute the m gammas --
%
for k=1:m
    kk = k - M - 1;
    w = kk*pi;
    sarg = v - w;
    if abs(sarg) > eps;
       val(k) = sin(sarg) / sarg;
    else;
       val(k) = 1;
    end;
end;
%
% Next, compute the left end basis:
rho = exp(u);
val1 = 1/(1+rho);
for k=2:m;
    kk = k - M - 1;
    val1 = val1 - (1/(1+exp(kk*h)))*val(k);
end;
%
% Finally, compute the right end basis --
%
valm = rho/(1+rho);
for k=1:m-1;
    kk = k - M - 1;
    valm = valm - (exp(kk*h)/(1+exp(kk*h)))*val(k);
end;
%
val(1) = val1;
val(m) = valm;
%


       
    
