% 
%  `conv_vec.m'
%
%  % This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
function [array] = conv1_vec(X,FF,gg)
%
%% Here X is externally defined in terms
% of an indefinite integration matrix A
% for integration from a to x, with
% by A =  h I^{(-1)} D(1/phi')
% = X S X^{-1}, where S is a diagonal
% matrix with entries
% (s_{-M}\,, \ldots\,,\ s_N), or if
% for integration from x to b, when
% B =  h (I^{(-1)})^T D(1/phi')
% = X S X^{-1}, and if F is the "Laplace
% transform" of f; so that FF is an
% externall defined diagonal matrix
% with j_th diagonal element F(s_j).  Also,
% gg = (g_(z_{-M}), ..., g(z_M))^T is an
% externall defined column vector. 
%
% Then V{int_a^x f(x-t) g(t) dt}
% approx F(A) gg = X FF X^{-1} gg, while
% V{int_x^b f(t-x) g(t) dt}
% approx F(B) gg = X FF X^{-1} gg.
%
% For example, consider the approximation 
% of the integral  q(x) = int_x^{infinity}
% exp(-t) t^{-1/3} dt
% In this case, f(x) = exp(-x), so that 
% F(s) = s/(1+s), and g(x) = x^{-1/3}. 
% 
% Then the following program gets an 
% an approximation to the vector 
% qq = V(q) for approximating q on 
% (0, infinity): 
%
% M = 10;
% N = 5;
% m = M+N+1;
% h = pi/sqrt(N);
% z = Sincpoints3(M,N,h);
% A = Int3_m(M,N,h);
% [X S] = eig(A);
% s = diag(S);
% for j=1:m
%     F(j) = s(j)/(1 + s(j));
%     gg(j) = z(j)^(-1/3); 
% end
% gg = gg.';
% FF = diag(F);
% qq = conv1_vec(X,FF,gg);
% 
%
hh = inv(X)*gg;
gg = FF*hh;
array = X*gg;
