function [array] = conv3_vec(X,m1,Y,m2,Z,m3,G,g)
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% Here A = X*S*X^{-1}, (m1 x m1), 
% B = Y*T*Y^{-1}, (m2 x m2) and 
% C = Z*U*Z^{-1} are integration matrixes
% defined as in Sec. 2.5.2, while F and g 
% (both (m1 x m2 x m3)) are externally 
% defined matrices, based on Algorithm 2.5.2.
% See  Sec. 2.5.2. 
% That is, before entry into this 
% routine, we would first form A,B and C, 
% according to Sec. 2.5.2, 
% and then we would form X,S,Y,T and 
% Z,U via 
% [X S] = eig(A); 
% [Y T] = eig(B); 
% [Z U] = eig(C); 
% and then enter this routine.
% Here g is a matrix with (i,j,k)th
% element gg(x_i,y_j,z_k), with 
% x_i ,y_j and z_k denoting Sinc 
% points.  Then,   
% v(x,y,z) = int_a^x int_y^d 
% f(x-x',y-y',z-z') gg(x',y',z')dy'dx'dz' 
% (there lctually be 8 such possible 
% convolutions) at the array of Sinc 
% points,  i.e., the (i,j,k,l)th element 
% of  V then approximates 
% v(w_i,x_j,y_k,z_l) .
%
% The present algorithm uses the 
% algorithm conv2_vec2.m .
%
Xi = inv(X);
%
for j=1:m2
    for k=1:m3
        h(:,j,k) = Xi*g(:,j,k);
    % h(ii,j,k)
    end
end
%
clear g
%
for ii=1:m1
   for j=1:m2
      for k=1:m3
         hh(j,k,ii) = h(ii,j,k);
         GG(j,k,ii) = G(ii,j,k);
      end
   end
   H(:,:) = hh(:,:,ii);
   F(:,:) = GG(:,:,ii);  
   Q = conv2_vec(Y,m2,Z,m3,F,H);
   for j=1:m2
      for k=1:m3
         q(ii,j,k) = Q(j,k);
      end
   end
end
%
clear F
clear H
clear Q
%
%
for j=1:m2
    for k=1:m3
       V(:,j,k) = X*q(:,j,k);
    end
end
%
clear q
array = V;
clear V
%

