%
function [p]= conv_nl_discrete_p(x,z,wz,A,ker_p)
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% This program carries out the nonlinear convolution
% to approximate the integral p(x) of (1.5.61). 
% We assume the that we have as input the m_z x m_x 
% array [ker_p(k,l)] with ker_p(k,l) = k(z_k,x_l), 
%  where the $x = [x_l]$ are Sinc points of (a,b), and the 
%  z = [z_k] are Sinc points of (0,c), with c (>=) b-a, 
% these, as well as the weights wz and the indefinite 
% integration matrix A of order mx for Sinc indefinite 
% numerical integration over (a,x), with x in (a,b) are 
% given.
% 
% At the outset we determine the eigenvalues s_j and the 
% corresponding matrix X  of eigenvectors of A.  
% We then approximate of K(s_j,z_l) by integration of the 
% integral (1.5.62) for every s = s_j and t = x_l using 
% Sinc quadrature.  
%
% Finally, we apply the formula on the extreme 
% right hand side of (1.5.64) to approximate p(x) 
% at the Sinc points x_j.
% 
m1 = size(x);
m2 = size(z);
[X S] = eig(A);
s = diag(S);
Xi = inv(X);
%
% Laplace transform of k(x,t)
%
K = zeros(m1,m2);
for l=1:m1
    for k=1:m1
        for j=1:m2
            K(k,l) = K(k,l) + wz(j)*ker_p(j,l)*exp(-z(j)/s(k));
        end
    end
end
%
% Approximating p(x_j)
%
for k=1:m1
    v(k) = 0;
    for l=1:m1
        v(k) = v(k) + Xi(k,l)*K(k,l);
    end
end
%
p = v*X.';


  

    
            
        
