%
% `convnl1_q.m'
% 
function [array] = convnl1_q(X,KK)
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% This routine computes a vector of values
% of  q(x) at the m Sinc points z_j of 
% either 
%
%  (1)  q(x) = \int_a^x k(x,x-y) dy
%
% or 
%
%  (2)  q(x) = \int_x^b k(x,y-x) dy
%
% at the sinc-points x = z_j.  See the 
% algorithm for q(x) in Equation (1.5.68)
% of Sec. 1.5.11 of the handbook.
%
% Here (a,b) is either the real line, 
% or a subinterval of the real line.
%
% If (a,b) corresponds to the  
% transformation # k of Sec. 1.5.3, then 
% we must use the routine A = Intk_p.m for 
% the integral (1),   and A = Intk_m.m for
% the integral (2).  It is presumed, of 
% course, that Sinc points z_1, ..., z_m
% are already known, or that they can 
% be determined in the usual way.
%
% One determines X and S externally, 
% where S = diag(s_1, ..., s_m) is a 
% diagonal matrix, such that A = X S X^{-1},
% and where X and S can, e.g., be determined
% via the command  [X S] = eig(A).
%
% It is furthermore assumed that the m x m 
% matrix KK, where KK(j,k) = K(z_j, s_k), 
% is determined externally; this corresponds 
% to either knowing explicitly the "Laplace 
% transform"
% 
% K(x,s) = \int_0^c k(x,t) exp(-t/s) dt
%
% where c > (b-a), or being able to compute 
% it. 
%
% This same routine also works for 
% computing the integral (2) above. 
% with (of course) a different choice of
% A.
%
m = length(X);
o = ones(m,1);
y = inv(X)*o;
for j=1:m
    qq = 0;
    for l=1:m
        qq = qq + KK(j,l)*y(l)*X(j,l);
    end
    q(j) = qq; 
end
array = q;

