function [array] = convnl3_vec(X,m1,Y,m2,Z,m3,F,g)
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% For approximating integrals of the form 
% of (2.5.46) of the handbook, i.e., 
%
% U(x,y,z) = \int\int\int_B 
%
% f(x-x',y-y',z-z') g(x',y',z') dz' dy' dx',
%
% with
% 
% B = {(x,y,z): a1 < x < b1, a2(x) < y < b2(x), 
%
        a3(x,y) < z < b3(x,y)}.
%
% Our inouts are actually for solving the 
% transformed form of (2.5.46), i.e., (2.5.47), 
% with use of the 3-d Laplace transform (2.5.48).
%
% To this end, using the notations 
% (xe_i\,,et_j\,,ze_k) for Sinc points e^w/(1+e^w)
%   with w = i\,h_1, j\,h_2, and k\,h_3 resp.,
% r_i\,, s_j\,, t_k eigenvalues of indefinite 
% integration matrices (see (2.5.42) 
% be <=> beta = b_1-a_1, 
% ga_i <=> b_2(xe(i))-a_2(xe(i)), de_{i,j} 
% <=> b_3(xe(i),ye(j))- a_3(xe(i),ye(j)) 
% and Laplace transform $F$ as defined in 
% (2.5.48), we assume as inputs the matrices 
% X, Y, and Z of size m_1, m_2 and m_3 resp., as 
% well as the inputs 
% F(i,i',j,j',k) < = > 
% F(be r_i, ga_{i'}s_j, de_{i',j'} te_k)
% and
% g(i',j',k') 
% <=> g(a_1+be xe_i',a_2(xe(i')+ ga_{i'}et_j',...
% ... a_3(xe(i'),et(i',j')+de(i',j')ze(k')
%
% This routine is the same for all 8 indefinite
% integrals, which are evaluated via
% the routine described in Algorithm 2.5.6 of 
% the handbook.  
%
% Form 
%
Xi = inv(X); Yi = inv(Y); Zi = inv(Z);
%
% Eliminate k' by taking the following w.r.t. k' 
%
hh(i',j,k')*Zi(k,k');
%
% Hadamard product:
%
% Set 
GG(i',i,j,j',k) = F(i,i',j,j',k)*hh(i',j',k); 
% 
% Tensor product w.r.t. i' to eliminate i'
%
HH(i,j,k,j') = Xi(i,i')*GG(i,i',j,j',k);
%
% Eliminae j' via tensor product with 
% respect to j'
%
    KK(i,j,k) = HH(i,j,k,j')*(Yi(j,j')).';
%
% Form LL(l,j,k) bia a tensor product w.r.t. i
%
LL(l,j,k) = X(l,i)*KK(i,j,k);
% 
% Set 
 U(l,:,:) = Y*LL(l,:,:)Z.';
