
%
% `ex4_lap_inv.m'
%
% This program is copyrighted by SINC, LLC
% an belongs to SINC, LLC
%
% This is an example of Laplace 
% transform inversion, via 
% Sinc-Pack.  See Sec. 1.5.10.
%
% Here we start with the given
% Laplace transform,
% 
% g(s) = hat(f)(s) 
% = int_0^infty f(t) exp(-st) dt
% = (pi/2)^{1/2}/(w(s)(1 + w(s))^{1/2})
% 
% 
% with
%
% w(s) = (1 + s^2)^{1/2},
% 
% and we wish to determine a function
% f(t) on (0,infinity) which gives rise 
% to this transform.  We will also
% compare with the exact solution
% 
% f(t) = t^{-1/2} sin(t).
%
% A "usual" start:
%
clear all
%
M = 15;
N = 20;
m = M+N+1;
h = pi/sqrt(N);
%
% Sincpoints and indefinite
% integration matrix for the
% interval (0,infinity) based on the 
% transformation #4 of Sec. 1.5.3 
%
z = Sincpoints4(M,N,h);
A = Int4_p(M,N,h);
[X S] = eig(A);
%
s = diag(S);
%
const = sqrt(pi/2);
for j=1:m
   p = s(j);
   w = sqrt(1 + p^2);
   gg(j) = const*p^(1/2)/(w*sqrt(1+w));
end
%
G = diag(gg);
%
f_approx = real(Lap_inv(X,G,m));
%
for j=1:m
    t = z(j);
    f_exact(j) = t^(-1/2)*sin(t);
end
%
plot(z,f_exact,'-',z,f_approx,'o')
%
pause
%
%figure(1)
title('Lapace Transform Inversions')
xlabel('Sinc Points')
ylabel('Inversions')
print -dps2 Lap_inv.ps
%
pause
% Plotting over the interval (0,4 pi):
step = 4*pi/100;
for j=1:100
    xx(j) = (j-1/2)*step;
    u = phi4(xx(j));
    v = basis(u,M,N,h);
    ffa(j) = v*f_approx;
    ffe(j) = v*f_exact.';
end
%
plot(xx,ffe,'-',xx,ffa,'.')
pause
% figure(2)
title('Sinc Interpolated Laplace Transform Inversion')
xlabel('x-Values')
ylabel('Inversions')
print -dps2 Lap_inv_interp.ps
pause
plot(xx,ffe-ffa,'-')
pause
% figure(3)
title('Sinc Interpolated Error')
xlabel('x-Values')
ylabel('Error')
print -dps2 Lap_inv_err.ps

