%
% ex_abel.m'  
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% Example of Application of Sinc Convolution 
% 
%  Here we want to compute approximations to the 
%  integrals 
%
% Exact1(x) 
% = \int_0^x (x-t)^{-1/2} t^{-1/2} dt  
% = \pi  
%
% and 
% 
%  Exact2(x) 
%  = \int_x^1 (t-x)^{-1/2} t^{-1/2} dt  
%
%  = \log((1 + (1-x)^{1/2})/(1 - (1-x)^{1/2})).
%
clear all
M = 16;
N = 8;
m = M+N+1;
Ei = Im(m); % This is the m x m I^{-1} matrix.
h = sqrt(2/N);
% 
% Computation of sinc-points, weights, 
% g(x) = x^{-1/2}, and exact solution at Sinc 
% points.
%
z = Sincpoints2(0,1,M,N,h);
%
% The function x^{-1/2} evaluated at the Sinc 
%  points.
%
for j=1:m
   g(j) = 1/sqrt(z(j));
%
% The function (1-x)^{1/2} evaluated at the 
% Sinc points.
%
   sq = sqrt(1 - z(j));
%
% The exact value of the first convolution.
%
   exact1(j) = pi;
%
% The exact value of the second convolution.
%
   exact2(j) = log((1+sq)/(1-sq));
end
%
g = g.';
exact1 = exact1.';
exact2 = exact2.';
%
% The Sinc Indefinite integration matrix for 
% integration from 0 to x.
%
A = Int2_p(0,1,M,N,h);
%
% The Sinc Indefinite integration matrix for 
% integration from x to 1.
%
B = Int2_m(0,1,M,N,h);
%
% Eigenvector and eigenvalue matrix 
% decomposition of A .
%
[X S] = eig(A);
% S
% Eigenvector and eigenvalue matrix 
% decomposition of B .
%
[Y T] = eig(B);
Xi = inv(X);
Yi = inv(Y);
%
% The "Laplace transform" of f(t) = t^{-1/2} is 
%
%  F(s) = (pi*s)^{1/2}.
%
for j=1:m
  s(j) = sqrt(pi*S(j,j));
end
%
s = s.';
%
% The next matrix, S, is a diagonal matrix, with 
% j^{th} diagonal entry s(j).
%
S = diag(s);
A = X * S * Xi; % This A is now  F(A).
B = Y * S * Yi; % This B is now  F(B).
p = real(A*g);  
% This is the Sinc approximation of 
% exact1(z(i)) at the Sinc points.
q = real(B*g);  
% This is the Sinc approximation of 
% exact2(z(i)) at the Sinc points.
plot(z,exact1,'o',z,p)
pause;
plot(z,exact2,'o',z,q)
%



