%
% `lap_poi_disc.m'
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% We solve here the problem 
%
% nabla u = -ff in D, 
%
% with D the unit disc, and with 
% ff = 4; 
% subject to the boundary conditions 
% u =  1 on C_+ 
% and 
% u =  - 1 on C_-.  
%
% Here C_+ (resp. C_-) denotes the 
% boundary of D in the upper (resp., 
% lower) half of the complex plane.  
% The exact solution to this problem 
% is 
% 
% u = 1 - x^2 - y^2 +  
%     + 2/pi*tan^{-1}(2y/(1-x^2-y^2)).
%  
% The method of solution is to use 
% the free space Green's function 
%
% G(x,y) = (4 pi)^{-1} log(x^2 + y^2)
%
% to produce a particular solution 
%
% U(x,y) 
% = \int\int_D G(x-p,y-q) f(p,q) dp dq
% 
% via use of the Sinc convolution 
% algorithm `convnl2_vec.m' and then 
% to use a different Green's function
% (since it is available) -- the 
% Poisson kernel -- to produce a 
% function  v  which is harmonic in 
% D and which satisfies the boundary 
% condition 
%  
% v(x,y) =  1 -  on C_+ 
% 
% and 
%
% v(x,y) = - 1  on C_-.  
% 
% Then u = U + v solves the 
% above Poisson-Dirichlet problem.
%
clear all
M1 = input('Enter Sinc point number, M1:   ')
N1 = input('Enter Sinc point number, N1:   ')
M2 = input('Enter Sinc point number, M2:   ')
N2 = input('Enter Sinc point number, N2:   ')
m1 = M1+N1+1;
m2 = M2+N2+1;
h1 = 0.45/sqrt(N1);
h2 = 0.45/sqrt(N2);
format long e
%
% We provide here the explicit 
% colmputation of the zeros 
% z_k = (e^{kh}-1)/(e^{kh}+1) for 
% (-1,1) using the formula 
% 1-z_k^2 = 4e^{kh}/(1+e^{kh})^2
% instead of computing (1-z_k^2) after z_k 
% is cpomputed,, since this latter method 
% is not a good procedure to use in the 
% case when z_k is close to 1.
%
  al = 2;
TT = exp(h1);
TM1 = 1/TT^M1;
for j=1:m1
   T2 = sqrt(TM1);
   den = 1+TM1;
   bet(j) = 4*T2/den;
   z1(j) = TM1/den;
   w1(j) = h1*TM1/den^2;
   TM1 = TM1*TT;
end
%
TT = exp(h2);
TM2 = 1/TT^M2;
for k=1:m2
    den = 1 + TM2;
    z2(k) = TM2/den;
    w2(j) = h2*TM2/den^2;
    TM2 = TM2*TT;
end
% Computing the right hand side 
% vector ff
%
for ii = 1:m1
  for j = 1:m2
   ff(ii,j) = 4;
  end
end
%
% Construction and diagonalization 
% of indefinite integration matrices 
% Aj and Bj, j=1,2 <=> x,y
%
A1 = Int2_p(0,1,M1,N1,h1);
B1 = Int2_m(0,1,M1,N1,h1);
A2 = Int2_p(0,1,M2,N2,h2);
B2 = Int2_m(0,1,M2,N2,h2);
%
[X1 S] = eig(A1);
[Y1 S] = eig(B1);
[X2 T] = eig(A2);
[Y2 T] = eig(B2);
clear A1
clear B1
clear A2
clear B2
%
s = diag(S);
clear S
sig = diag(T);
clear T
% 
% Now, the "Laplace transform", as 
% given in Step #6 of Algorithm 2.1.2
% and using the routine lap_tr_poi2.m.
%
gamma = 0.5772156649015328606065;
%
for ii=1:m1
    p = al*s(ii);
    for k=1:m2
        for j=1:m1
            q = bet(j)*sig(k); 
            F(ii,j,k) = 1/(1/p^2 + 1/q^2)*(-1/4 + ...
            1/(2*pi)*(q/p*(gamma-log(q)) + ...
            p/q*(gamma-log(p))));
        end
    end
end
U = zeros(m1,m2);
for n=1:4
   if n == 1
    A = X1; B = X2;
    elseif n == 2
    A = X1; B = Y2;
   elseif n == 3
     A = Y1; B = X2;
   else
     A = Y1;  B = Y2;
   end
U = U + convnl2_vec(A,m1,B,m2,F,ff);
end
%
U = real(U);
%
surf(z2,z1,U);
pause
%
% Now, compare with the exact solution, 
% U_ex(x,y) = 1-x^2-y^2.  But this must first
% be approximated on the square:
%
for j=1:m1
    for k=1:m2
        x = -1 + al*z1(j);
        y = -bet(j)/2+bet(j)*z2(k);
        U_ex(j,k) = 1 - x^2 -y^2;
    end
end
%
surf(z2,z1,U_ex)
pause
surf(z2,z1,U - U_ex)
pause
%
% L1 and L2 norms of error
%
Er1 = 0;
Er2 = 0;
for j=1:m1
    for k=1:m2
        eee = abs(U(j,k) - U_ex(j,k));
        Er1 = Er1 + w1(j)*w2(k)*eee;
        Er2 = Er2 + w1(j)*w2(j)*eee^2;
    end
end
%
L1error = Er1 % L1 norm
L2error = sqrt(Er2) % L2 norm
