
%========================================================================
% `lap_square.m'
%
% Solving Poisson problem 
%
% nabla u = -f in D = (-1,1) x (-1,1) 
%
% with f(x,y) = -2,
% 
% subject to 
%
%  u(x,y) = x^2 on the boundary of D.
%
% Our procedure is based on use of
% coupled SINC and BEM.  
%
clear all
N = input('Enter number of Sinc points:   ')
h = 2.5/sqrt(N);
m = 2*N+1;
format long e
%
% Sinc points and weights over (-1,1)
%
z = Sincpoints2(-1,1,N,N,h);
w = weights2(-1,1,N,N,h);
W = diag(w);
%
% Construction and diagonalization of 
% indefinite integration matrices 
%
A1 = Int2_p(-1,1,N,N,h);
B1 = Int2_m(-1,1,N,N,h);
[X S] = eig(A1);
Xi = inv(X);
[Y S] = eig(B1);
%
% Note: A1 and B1 have the same 
% eigenvalue matrix. 
%
Yi = inv(Y);
%
s = diag(S);
%
% I. The particular solution 
% 
% Computing ``Laplace transform'' of the 
% Green's function'' at the eigenvalues 
% of A1 and B1 -- see Lemma 2.4.2
%
gamma = 0.5772156649015328606065;
%
for j = 1:m
    p=s(j);
    for k = 1:m
        q=s(k);
        G(j,k) = 1/(1/p^2 + ...
1/q^2)*(-1/4 +1/(2*pi)*(q/p*(gamma-log(q))+ ...
            p/q*(gamma-log(p))));
   end
end
%
% Computing Right Hand Side --  matrix -- 
% f = [f(z_j\,,z_k)], with f(x,y) = 2.   
%
%
for j = 1:m
  for k = 1:m
   f(j,k) = 2;
  end
end

%
% Computing the Sinc Convolution of the 
% Green's Function  G  and f via use of Sinc 
% Convolution, based on Algorithm 2.5.1. 
%
% We thus get a particular solution of the 
% non-homogeneous PDE.
%
U = X*(G.*(Xi*f*Xi.'))*X.';
U = U + Y*(G.*(Yi*f*Xi.'))*X.';
U = U + X*(G.*(Xi*f*Yi.'))*Y.';
U = U + Y*(G.*(Yi*f*Yi.'))*Y.';
UR = real(U);
%
% Plot of UR
%
figure(1)
surf(z,z,UR)
print -dps2 dirich_pois_partic.ps
pause
%
% II.  Solving the Homogenous Problem
%
% We next need to compute a solution of the 
% homeogenous PDE to satisfy the boundary 
% conditions.  This is done via BIE, i.e., via 
% the procedure described above.
%
% Constructing right-hand side vectors
%
for j = 1:m
    g2(j) =  z(j)^2 - UR(j,m);
    g4(j) =  z(j)^2 - UR(j,1);
  end
% 
for j=1:m
    g1(j) = z(m)^2 - UR(m,j);
    g3(j) = z(1)^2 - UR(1,j);
end
%
% Setting up the blocks A, B, C
%
for j = 1:m
    for k = 1:m
       A1(j,k) = (1-z(j))/pi/((1-z(j))^2 + (1+z(k))^2);
       A2(j,k) = 2/pi/(4 + (z(j) + z(k))^2);
       A3(j,k) = (1+z(j))/pi/((1+z(j))^2 + (1-z(k))^2);
    end
end
%
A1 = A1*W;
A2 = A2*W;
A3 = A3*W;
D = eye(m);
%
% Now use the above created matrices and vector 
% to create the `big system' of equations.
%
A = [D A1 A2 A3; A3 D A1 A2; ...
        A2 A3 D A1;  A1 A2 A3 D];
g = [g1 g2 g3 g4].';
%
% Solving linear equations
%
r = A\g;
%
% Approximate solution via Harmonic Sinc
% 
     r1 = r(1:m);
     r2 = r(m+1:m+m);
     r3 = r(m+m+1:m+m+m);
     r4 = r(m+m+m+1:m+m+m+m);
%
% Solution computed on Sinc grid
%
p = zeros(m,m);
%
for j=1:m
    for k=1:m
        zz = z(j) + i*z(k);
        u1 = phi2(1-i,1+i,zz);
        u2 = phi2(1+i,-1+i,zz);
        u3 = phi2(-1+i,-1-i,zz);
        u4 = phi2(-1-i,1-i,zz);
        p1 = basis_harm(u1,N,N,h);
        p2 = basis_harm(u2,N,N,h);
        p3 = basis_harm(u3,N,N,h);
        p4 = basis_harm(u4,N,N,h);
        p(j,k) = p1*r1 + p2*r2 + p3*r3 + p4*r4;
    end
end
% Approximte solution
% 
 v = p+UR;
%
% Plots of Homogenous & Complete Sols
%
figure(2)
surf(z,z,p); 
print -dps2 dirich_lap.ps
pause
%
figure(3)
surf(z,z,v); 
print -dps2 dirich_tot.ps
