%
% `navier_stokes1.m'
%
% This program is copyrighted by SINC, LLC
% an belongs to SINC, LLC
% 
% Sinc Solution of (4.3.7) -- (4.3.9)
% as outlined in Sec. 2.4.6.
% 
% Initializations:
%
clear all
%
tic
Eps = 0.2;
%
T = 0.00008;
%
N = 8;
m = 2*N+1;
hs = 1/sqrt(N);
ht =0.6*hs; 
zs = Sincpoints1(N,N,hs); % Spacial Sinc pts.
zt = Sincpoints2(0,T,N,N,ht); % Time Sinc pts. on (0,T)
%
% Diagonalization of both spacial indefinite 
% integration matrices:
%
[X,Y,SS] = Int_both1(N,N,hs);
Xi = inv(X);
Yi = inv(Y);
s = diag(SS);
It = Int2_p(0,T,N,N,ht); % Time integration matrix on (0,T)
% It = Int3_p(N,N,ht); % Time integration matrix on (0,infty)
% It = Int4_p(N,N,ht); % Time integration matrix on (0,infty)
% Dr = Domega1_1(N,N,hs); % Spacial derivative matrix
Dr = Dpoly1_1(N,N,hs); % Spacial derivative matrix
% Dt = (Domega2_1(0,T,N,N,ht)).'; % Tr(Time derivative matrix)
Dt = (Dpoly2_1(0,T,N,N,ht)).'; % Tr(Time derivative matrix)
% Dt = (inv(It)).';
[Z,TT] = eig(It);
Zi = inv(Z);
tt = diag(TT);
clear SS TT It
%
% The initial velocity vector, u^0:
%
[U01,U02,U03] = navier_stokes_ic(zs);
%
for ii = 1:m
    for j=1:m
        for k=1:m
            for l=1:m
                V1(ii,j,k,l) = U01(ii,j,k);
                V2(ii,j,k,l) = U02(ii,j,k);
                V3(ii,j,k,l) = U03(ii,j,k);
            end
        end
    end
end
%
clear U01 U02 U03
%
% We first compute the "Laplace transforms"
% of the Green's functions
%
% First, the 4 dim. "Lap. tr."
% of the function in (2.4.61),
% for the case of n = 3; see
% Theorem 2.4.13 (b).
%
for ii = 1:m
    for j = 1:m
        for k = 1:m
            for l = 1:m
  R(ii,j,k,l) = lap_tr_heat4(s(ii),s(j),s(k),tt(l),Eps); 
            end
        end
    end
end
% 
% It is convenient to first compute the 
% "Laplace transform" G0 of the three 
% dimensional Pisson Green's function,
% before computing the "Laplace transform"
% for the operator S. The function G0 is 
% used for computing the pressure, after 
% convergence of the iterative process 
% for the vector (U1, U2, U3).
%
for ii = 1:m
    for j = 1:m
        for k = 1:m  
            G0(ii,j,k) = lap_tr_poi3(s(ii),s(j),s(k));
        end
    end
end
%
% It is now convenient to compute S 
% see Theorem 2.4.13 (c)
% 
for l = 1:m
    S(:,:,:,l) = (G0(:,:,:) - Eps*R(:,:,:,l))*tt(l);
end
%
% Determination of the operation Q(U0)
%
% First determine R(U0), then differentiate wrt t
%
U01 = zeros(m,m,m,m);
U02 = U01;
U03 = U01;
for n = 1:8
    if n == 1
     A = X; B = X; C = X;
    elseif n == 2
     A = X; B = X; C = Y;
    elseif n == 3
     A = X; B = Y; C = X;
    elseif n == 4
     A = X; B = Y; C = Y;
    elseif n == 5
     A = Y; B = X; C = X;
    elseif n == 6
     A = Y; B = X; C = Y;
    elseif n == 7
     A = Y; B = Y; C = X;
    else 
     A = Y; B = Y; C = Y;
    end
    U01 = U01 + real(conv4_vec(A,m,B,m,C,m,Z,m,R,V1));
    U02 = U02 + real(conv4_vec(A,m,B,m,C,m,Z,m,R,V2));
    U03 = U03 + real(conv4_vec(A,m,B,m,C,m,Z,m,R,V3));
end
%
clear V1 V2 V3
%
for ii=1:m
    for j=1:m
        for k=1:m
            for l=1:m
                W1(l) = U01(ii,j,k,l);
                W2(l) = U02(ii,j,k,l);
                W3(l) = U03(ii,j,k,l);
            end
            DW1 = W1*Dt;
            DW2 = W2*Dt;
            DW3 = W3*Dt;
            for l=1:m
	        U01(ii,j,k,l) = DW1(l);
	        U02(ii,j,k,l) = DW2(l);
	        U03(ii,j,k,l) = DW3(l);
            end
        end       
    end
end
%
clear W1 W2 W3 DW1 DW2 DW3 Dt
% Start of Iteration Scheme
%
kkk = 0;
er = 1;
%
% Initial values of iterates
%
V1 = U01;
V2 = U02;
V3 = U03;
U1 = U01;
U2 = U02;
U3 = U03;
%
while (er > 10^(-4))
%
% Dervivatives of flows and sums of 
% nonlinear terms.  First the 
% derivatives
%
for l=1:m
    for j=1:m
        for k=1:m
            for ii=1:m
                W1(ii) = U1(ii,j,k,l);
                W2(ii) = U2(ii,j,k,l);
                W3(ii) = U3(ii,j,k,l);
            end
            DW1 = Dr*W1.';
            DW2 = Dr*W2.';
            DW3 = Dr*W3.';
            for ii=1:m
                U11(ii,j,k,l) = DW1(ii);
                U21(ii,j,k,l) = DW2(ii);
                U31(ii,j,k,l) = DW3(ii);
            end
        end
    end
    for ii=1:m
        for k=1:m
            for j=1:m
                W1(j) = U1(ii,j,k,l);
                W2(j) = U2(ii,j,k,l);
                W3(j) = U3(ii,j,k,l);
            end
            DW1 = Dr*W1.';
            DW2 = Dr*W2.';
            DW3 = Dr*W3.';
            for j=1:m
                U12(ii,j,k,l) = DW1(j);
                U22(ii,j,k,l) = DW2(j);
                U32(ii,j,k,l) = DW3(j);
            end
        end
    end
    for ii=1:m
        for j=1:m
            for k=1:m
                W1(k) = U1(ii,j,k,l);
                W2(k) = U2(ii,j,k,l);
                W3(k) = U3(ii,j,k,l);
            end
            DW1 = Dr*W1.';
            DW2 = Dr*W2.';
            DW3 = Dr*W3.';
            for k=1:m
                U13(ii,j,k,l) = DW1(k);
                U23(ii,j,k,l) = DW2(k);
                U33(ii,j,k,l) = DW3(k); 
            end
        end
    end
end
%
clear W1 DW1 W2 DW2 W3 Dw3
%
% R nonlinear terms
%
rr1 = U1.*U11+U2.*U12+U3.*U13;
rr2 = U1.*U21+U2.*U22+U3.*U23;
rr3 = U1.*U31+U2.*U32+U3.*U33;
%
% S nonlinear terms
%
% First the function g
%
g = U11.*U11 + U22.*U22 + U33.*U33 + ...
    2*(U12.*U21 + U13.*U31 + U23.*U32);
%
% Next the three first derivatives of g
%
%
% Derivatives of g
%
for l=1:m
    for j=1:m
        for k=1:m
            for ii=1:m
                W1(ii) = g(ii,j,k,l);
            end
            DW1 = Dr*W1.';
            for ii=1:m
                g1(ii,j,k,l) = DW1(ii);
            end
        end
        for ii=1:m
            for k=1:m
                W1(k) = g(ii,j,k,l);
            end
            DW1 = Dr*W1.';
            for k=1:m
                g3(ii,j,k,l) = DW1(k);
            end
        end
    end
       for ii=1:m
           for k=1:m
               for j=1:m
                   W1(j) = g(ii,j,k,l);
               end
               DW1 = Dr*W1.';
               for j=1:m
                   g2(ii,j,k,l) = DW1(j);
               end
           end
       end
end
%
clear W1 DW1
%
% Convolution for operator R
%
U1 = U01;
U2 = U02;
U3 = U03;
for n = 1:8
    if n == 1
     A = X; B = X; C = X;
    elseif n == 2
     A = X; B = X; C = Y;
    elseif n == 3
     A = X; B = Y; C = X;
    elseif n == 4
     A = X; B = Y; C = Y;
    elseif n == 5
     A = Y; B = X; C = X;
    elseif n == 6
     A = Y; B = X; C = Y;
    elseif n == 7
     A = Y; B = Y; C = X;
    else 
     A = Y; B = Y; C = Y;
    end
    U1 = U1 + real(conv4_vec(A,m,B,m,C,m,Z,m,R,rr1));
    U2 = U2 + real(conv4_vec(A,m,B,m,C,m,Z,m,R,rr2));
    U3 = U3 + real(conv4_vec(A,m,B,m,C,m,Z,m,R,rr3));
end
%
clear rr1 rr2 rr3
%
% Conmvolution evaluation of operation S 
%
for n = 1:8
    if n == 1
     A = X; B = X; C = X;
    elseif n == 2
     A = X; B = X; C = Y;
    elseif n == 3
     A = X; B = Y; C = X;
    elseif n == 4
     A = X; B = Y; C = Y;
    elseif n == 5
     A = Y; B = X; C = X;
    elseif n == 6
     A = Y; B = X; C = Y;
    elseif n == 7
     A = Y; B = Y; C = X;
    else 
     A = Y; B = Y; C = Y;
    end
    U1 = U1 + real(conv4_vec(A,m,B,m,C,m,Z,m,S,g1));
    U2 = U2 + real(conv4_vec(A,m,B,m,C,m,Z,m,S,g2));
    U3 = U3 + real(conv4_vec(A,m,B,m,C,m,Z,m,S,g3));
end
%
clear A B C g1 g2 g3
%
ERMAT = U1-V1+U2-V2+U3-V3;
ERVEC = reshape(ERMAT,m^4,1);
err = norm(ERVEC);
clear ERMAT ERVEC
%
V1 = U1;
V2 = U2;
V3 = U3;
%
kkk = kkk+1;
Iter_No_k = kkk
Iter_Error_k = err
er = err;
end
%
% At this point we have computed the 
% vector solution to Navier-Stokes 
% equations.  Now, for the pressure:
%
q = zeros(m,m,m);
%
for l=1:m
    AA = g(:,:,:,l);
    q = q + real(conv3_vec(X,m,X,m,X,m,G0,AA));
    q = q + real(conv3_vec(Y,m,X,m,X,m,G0,AA));
    q = q + real(conv3_vec(X,m,X,m,Y,m,G0,AA));
    q = q + real(conv3_vec(Y,m,X,m,Y,m,G0,AA));
    q = q + real(conv3_vec(X,m,Y,m,X,m,G0,AA));
    q = q + real(conv3_vec(Y,m,Y,m,X,m,G0,AA));
    q = q + real(conv3_vec(X,m,Y,m,Y,m,G0,AA));
    q = q + real(conv3_vec(Y,m,Y,m,Y,m,G0,AA));
    p(:,:,:,l) = q(:,:,:);
end
%
toc
%
%
% Printouts
%
figure(1)
surf(zs,zs,U2(:,:,N+1,N+1))
pause
print -dps2 n_s1_U2xy_0_halfT.ps
%
for ii=1:m
    for j=1:m
        U1T(ii,j) = U1(ii,N+1,N+1,j);
        U2T(ii,j) = U2(ii,N+1,N+1,j);
        U3T(ii,j) = U3(ii,N+1,N+1,j);
        pT(ii,j) = p(ii,N+1,N+1,j);
    end
end
%
figure(2)
surf(zt,zs,U2T)
pause
print -dps2 n_s1_U2x00t.ps
%
figure(3) 
surf(zs,zs,p(:,:,N+1,N+1))
pause
print -dps2 n_s1_pxy_0_halfT.ps
%
figure(4) 
surf(zt,zs,pT)
pause
print -dps2 n_s1_px00t.ps
%
