%
% `navier_stokes2.m'
%
% This program is copyrighted by SINC, LLC
% an belongs to SINC, LLC
% 
% Sinc Solution of (4.3.7) -- (4.3.9)
% as outlined in Sec. 2.4.6.
% 
% Initializations:
%
clear all
% 
% Start time
%
tic
%
% Eps = 0.2;
Eps = 0.5;
T = 0.1;
% T = 0.00008;
%
% Sinc parameters
%
N = 8;
m = 2*N+1;
hs = 1/sqrt(N);
ht = 0.6*hs;
zs = Sincpoints1(N,N,hs); % Spacial map #1 Sinc pts
zt = Sincpoints2(0,T,N,N,ht); % Time Sinc points
Dr = D1_1(N,N,hs);
%
% Diagonalization of both spacial indefinite 
% integration matrices:
%
[X,Y,SS] = Int_both1(N,N,hs);
% [X,Y,SS] = Int_both5(N,N,hs);
Xi = inv(X);
Yi = inv(Y);
s = diag(SS);
It = Int2_p(0,T,N,N,ht); % Time integration matrix on (0,T)
[Z,TT] = eig(It);
Zi = inv(Z);
tt = diag(TT);
clear SS TT It
%
% The initial velocity vector, u^0:
%
[U01,U02,U03] = navier_stokes_ic(zs);
%
% We first compute the Green's functions
% -- see the discussion following (4.3.11)
%
for ii = 1:m
    a = s(ii);
    for j = 1:m
        b = s(j);
        for k = 1:m
            c = s(k);
            G0(ii,j,k) = lap_tr_poi3(a,b,c);
            for l = 1:m
                t = tt(l);
Q(ii,j,k,l) = lap_tr_heat4(a,b,c,t,Eps)/t;
R = navier_stokes_R(a,b,c,t,Eps);
S = navier_stokes_S(a,b,c,t,Eps);
R2(j,k,ii,l) = R;
S2(j,k,ii,l) = S;
R3(k,ii,j,l) = R;
S3(k,ii,j,l) = S;
R1(ii,j,k,l) = R;
S1(ii,k,k,l) = S;
V1(ii,j,k,l) = U01(ii,j,k);
V2(ii,j,k,l) = U02(ii,j,k);
V3(ii,j,k,l) = U03(ii,j,k);
            end
        end
    end
end
%
clear U01 U02 U03 R S
%
% Determination of the vector Q^0
%
U01 = zeros(m,m,m,m);
U02 = U01;
U03 = U01;
for n = 1:8
    if n == 1
     A = X; B = X; C = X;
    elseif n == 2
     A = X; B = X; C = Y;
    elseif n == 3
     A = X; B = Y; C = X;
    elseif n == 4
     A = X; B = Y; C = Y;
    elseif n == 5
     A = Y; B = X; C = X;
    elseif n == 6
     A = Y; B = X; C = Y;
    elseif n == 7
     A = Y; B = Y; C = X;
    else 
     A = Y; B = Y; C = Y;
    end
    U01 = U01 + real(conv4_vec(A,m,B,m,C,m,Z,m,Q,V1));
    U02 = U02 + real(conv4_vec(A,m,B,m,C,m,Z,m,Q,V2));
    U03 = U03 + real(conv4_vec(A,m,B,m,C,m,Z,m,Q,V3));
end
%
clear Q  V1 V2 V3 A B C
%
% Start of Iteration Scheme
%
er = 1;
kkk = 0;
%
% Initial values of iterates
%
V1 = U01;
V2 = U02;
V3 = U03;
while(er > 10^(-4))
%
% First compute the "R-contributions"
%
U1c = U01;
U2c = U02;
U3c = U03;
%
B11 = V1.*V1;
%
U1c = U1c - conv4_vec(X,m,X,m,X,m,Z,m,R1,B11);
U1c = U1c + conv4_vec(Y,m,X,m,X,m,Z,m,R1,B11);
U1c = U1c - conv4_vec(X,m,X,m,Y,m,Z,m,R1,B11);
U1c = U1c + conv4_vec(Y,m,X,m,Y,m,Z,m,R1,B11);
U1c = U1c - conv4_vec(X,m,Y,m,X,m,Z,m,R1,B11);
U1c = U1c + conv4_vec(Y,m,Y,m,X,m,Z,m,R1,B11);
U1c = U1c - conv4_vec(X,m,Y,m,Y,m,Z,m,R1,B11);
U1c = U1c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R1,B11);
%
clear B11
%
B22 = V2.*V2;
%
U2c = U2c - conv4_vec(X,m,X,m,X,m,Z,m,R2,B22);
U2c = U2c + conv4_vec(X,m,Y,m,X,m,Z,m,R2,B22);
U2c = U2c - conv4_vec(X,m,X,m,Y,m,Z,m,R2,B22);
U2c = U2c + conv4_vec(X,m,Y,m,Y,m,Z,m,R2,B22);
U2c = U2c - conv4_vec(Y,m,X,m,X,m,Z,m,R2,B22);
U2c = U2c + conv4_vec(Y,m,Y,m,X,m,Z,m,R2,B22);
U2c = U2c - conv4_vec(Y,m,X,m,Y,m,Z,m,R2,B22);
U2c = U2c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R2,B22);
%
clear B22
%
B33 = V3.*V3;
%
U3c = U3c - conv4_vec(X,m,X,m,X,m,Z,m,R3,B33);
U3c = U3c + conv4_vec(X,m,X,m,Y,m,Z,m,R3,B33);
U3c = U3c - conv4_vec(X,m,Y,m,X,m,Z,m,R3,B33);
U3c = U3c + conv4_vec(X,m,Y,m,Y,m,Z,m,R3,B33);
U3c = U3c - conv4_vec(Y,m,X,m,X,m,Z,m,R3,B33);
U3c = U3c + conv4_vec(Y,m,X,m,Y,m,Z,m,R3,B33);
U3c = U3c - conv4_vec(Y,m,Y,m,X,m,Z,m,R3,B33);
U3c = U3c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R3,B33);
%
clear B33
%
B12 = V1.*V2;
%
U1c = U1c - conv4_vec(X,m,X,m,X,m,Z,m,R2,B12);
U1c = U1c + conv4_vec(X,m,Y,m,X,m,Z,m,R2,B12);
U1c = U1c - conv4_vec(X,m,X,m,Y,m,Z,m,R2,B12);
U1c = U1c + conv4_vec(X,m,Y,m,Y,m,Z,m,R2,B12);
U1c = U1c - conv4_vec(Y,m,X,m,X,m,Z,m,R2,B12);
U1c = U1c + conv4_vec(Y,m,Y,m,X,m,Z,m,R2,B12);
U1c = U1c - conv4_vec(Y,m,X,m,Y,m,Z,m,R2,B12);
U1c = U1c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R2,B12);
%
U2c = U2c - conv4_vec(X,m,X,m,X,m,Z,m,R1,B12);
U2c = U2c + conv4_vec(Y,m,X,m,X,m,Z,m,R1,B12);
U2c = U2c - conv4_vec(X,m,X,m,Y,m,Z,m,R1,B12);
U2c = U2c + conv4_vec(Y,m,X,m,Y,m,Z,m,R1,B12);
U2c = U2c - conv4_vec(X,m,Y,m,X,m,Z,m,R1,B12);
U2c = U2c + conv4_vec(Y,m,Y,m,X,m,Z,m,R1,B12);
U2c = U2c - conv4_vec(X,m,Y,m,Y,m,Z,m,R1,B12);
U2c = U2c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R1,B12);
%
clear B12
%
B13 = V1.*V3;
%
U1c = U1c - conv4_vec(X,m,X,m,X,m,Z,m,R3,B13);
U1c = U1c + conv4_vec(X,m,X,m,Y,m,Z,m,R3,B13);
U1c = U1c - conv4_vec(X,m,Y,m,X,m,Z,m,R3,B13);
U1c = U1c + conv4_vec(X,m,Y,m,Y,m,Z,m,R3,B13);
U1c = U1c - conv4_vec(Y,m,X,m,X,m,Z,m,R3,B13);
U1c = U1c + conv4_vec(Y,m,X,m,Y,m,Z,m,R3,B13);
U1c = U1c - conv4_vec(Y,m,Y,m,X,m,Z,m,R3,B13);
U1c = U1c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R3,B13);
%
U3c = U3c - conv4_vec(X,m,X,m,X,m,Z,m,R1,B13);
U3c = U3c + conv4_vec(Y,m,X,m,X,m,Z,m,R1,B13);
U3c = U3c - conv4_vec(X,m,X,m,Y,m,Z,m,R1,B13);
U3c = U3c + conv4_vec(Y,m,X,m,Y,m,Z,m,R1,B13);
U3c = U3c - conv4_vec(X,m,Y,m,X,m,Z,m,R1,B13);
U3c = U3c + conv4_vec(Y,m,Y,m,X,m,Z,m,R1,B13);
U3c = U3c - conv4_vec(X,m,Y,m,Y,m,Z,m,R1,B13);
U3c = U3c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R1,B13);
%
clear B13
%
B23 = V2.*V3;
%
U2c = U2c - conv4_vec(X,m,X,m,X,m,Z,m,R3,B23);
U2c = U2c + conv4_vec(X,m,X,m,Y,m,Z,m,R3,B23);
U2c = U2c - conv4_vec(X,m,Y,m,X,m,Z,m,R3,B23);
U2c = U2c + conv4_vec(X,m,Y,m,Y,m,Z,m,R3,B23);
U2c = U2c - conv4_vec(Y,m,X,m,X,m,Z,m,R3,B23);
U2c = U2c + conv4_vec(Y,m,X,m,Y,m,Z,m,R3,B23);
U2c = U2c - conv4_vec(Y,m,Y,m,X,m,Z,m,R3,B23);
U2c = U2c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R3,B23);
%
U3c = U3c - conv4_vec(X,m,X,m,X,m,Z,m,R2,B23);
U3c = U3c + conv4_vec(X,m,Y,m,X,m,Z,m,R2,B23);
U3c = U3c - conv4_vec(X,m,X,m,Y,m,Z,m,R2,B23);
U3c = U3c + conv4_vec(X,m,Y,m,Y,m,Z,m,R2,B23);
U3c = U3c - conv4_vec(Y,m,X,m,X,m,Z,m,R2,B23);
U3c = U3c + conv4_vec(Y,m,Y,m,X,m,Z,m,R2,B23);
U3c = U3c - conv4_vec(Y,m,X,m,Y,m,Z,m,R2,B23);
U3c = U3c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R2,B23);
%
clear B23
%
% The function g of (2.4.73)
%
% First the derivatives
%
for l=1:m
    for j=1:m
        for k=1:m
            for ii=1:m
                W1(ii) = V1(ii,j,k,l);
                W2(ii) = V2(ii,j,k,l);
                W3(ii) = V3(ii,j,k,l);
            end
            DW1 = Dr*W1.';
            DW2 = Dr*W2.';
            DW3 = Dr*W3.';
            for ii=1:m
                U11(ii,j,k,l) = DW1(ii);
                U21(ii,j,k,l) = DW2(ii);
                U31(ii,j,k,l) = DW3(ii);
            end
        end
    end
    for ii=1:m
        for k=1:m
            for j=1:m
                W1(j) = V1(ii,j,k,l);
                W2(j) = V2(ii,j,k,l);
                W3(j) = V3(ii,j,k,l);
            end
            DW1 = Dr*W1.';
            DW2 = Dr*W2.';
            DW3 = Dr*W3.';
            for j=1:m
                U12(ii,j,k,l) = DW1(j);
                U22(ii,j,k,l) = DW2(j);
                U32(ii,j,k,l) = DW3(j);
            end
        end
    end
    for ii=1:m
        for j=1:m
            for k=1:m
                W1(k) = V1(ii,j,k,l);
                W2(k) = V2(ii,j,k,l);
                W3(k) = V3(ii,j,k,l);
            end
            DW1 = Dr*W1.';
            DW2 = Dr*W2.';
            DW3 = Dr*W3.';
            for k=1:m
                U13(ii,j,k,l) = DW1(k);
                U23(ii,j,k,l) = DW2(k);
                U33(ii,j,k,l) = DW3(k);
            end
        end
    end
end
%
% Now g --- see (2.4.73)
%
g = U11.*U11 + U22.*U22 + U33.*U33 + ...
    2*(U12.*U21 + U13.*U31 + U23.*U32);
%
clear U11 U22 U33 U12 U21 U13 U31 U23 U32
%
U1c = U1c - conv4_vec(X,m,X,m,X,m,Z,m,R1,g);
U1c = U1c + conv4_vec(Y,m,X,m,X,m,Z,m,R1,g);
U1c = U1c - conv4_vec(X,m,X,m,Y,m,Z,m,R1,g);
U1c = U1c + conv4_vec(Y,m,X,m,Y,m,Z,m,R1,g);
U1c = U1c - conv4_vec(X,m,Y,m,X,m,Z,m,R1,g);
U1c = U1c + conv4_vec(Y,m,Y,m,X,m,Z,m,R1,g);
U1c = U1c - conv4_vec(X,m,Y,m,Y,m,Z,m,R1,g);
U1c = U1c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R1,g);
%
U2c = U2c - conv4_vec(X,m,X,m,X,m,Z,m,R2,g);
U2c = U2c + conv4_vec(X,m,Y,m,X,m,Z,m,R2,g);
U2c = U2c - conv4_vec(X,m,X,m,Y,m,Z,m,R2,g);
U2c = U2c + conv4_vec(X,m,Y,m,Y,m,Z,m,R2,g);
U2c = U2c - conv4_vec(Y,m,X,m,X,m,Z,m,R2,g);
U2c = U2c + conv4_vec(Y,m,Y,m,X,m,Z,m,R2,g);
U2c = U2c - conv4_vec(Y,m,X,m,Y,m,Z,m,R2,g);
U2c = U2c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R2,g);
%
U3c = U3c - conv4_vec(X,m,X,m,X,m,Z,m,R3,g);
U3c = U3c + conv4_vec(X,m,X,m,Y,m,Z,m,R3,g);
U3c = U3c - conv4_vec(X,m,Y,m,X,m,Z,m,R3,g);
U3c = U3c + conv4_vec(X,m,Y,m,Y,m,Z,m,R3,g);
U3c = U3c - conv4_vec(Y,m,X,m,X,m,Z,m,R3,g);
U3c = U3c + conv4_vec(Y,m,X,m,Y,m,Z,m,R3,g);
U3c = U3c - conv4_vec(Y,m,Y,m,X,m,Z,m,R3,g);
U3c = U3c + conv4_vec(Y,m,Y,m,Y,m,Z,m,R3,g);
%
U1 = real(U1c);
U2 = real(U2c);
U3 = real(U3c);
%
clear U1c U2c U3c
%
% Error test
%
ERMAT = U1-V1+U2-V2+U3-V3;
ERVEC = reshape(ERMAT,m^4,1);
err = norm(ERVEC);
clear ERMAT ERVEC
%
V1 = U1;
V2 = U2;
V3 = U3;
%
kkk = kkk+1;
Iteration_no = kkk
Iteration_error = err
er = err;
end
%
% At this point we have computed the
% vector solution to Navier-Stokes
% equations.  Now, for the pressure:
%
qc = zeros(m,m,m);
%
for l=1:m
    AA = g(:,:,:,l);
    qc = qc + conv3_vec(X,m,X,m,X,m,G0,AA);
    qc = qc + conv3_vec(Y,m,X,m,X,m,G0,AA);
    qc = qc + conv3_vec(X,m,X,m,Y,m,G0,AA);
    qc = qc + conv3_vec(Y,m,X,m,Y,m,G0,AA);
    qc = qc + conv3_vec(X,m,Y,m,X,m,G0,AA);
    qc = qc + conv3_vec(Y,m,Y,m,X,m,G0,AA);
    qc = qc + conv3_vec(X,m,Y,m,Y,m,G0,AA);
    qc = qc + conv3_vec(Y,m,Y,m,Y,m,G0,AA);
    pc(:,:,:,l) = qc(:,:,:);
end
%
clear AA qc
%
p = real(pc);
%
toc
%
clear pc
%
% Printouts
%
% In what follows we produce several 
% surface plots of solutions obtained 
% at Sinc points. Some of these have 
% the appearance of not being very 
% accurate, whereas these solutions 
% are, in fact quite accurate at the 
% Sinc points.  We have thus also 
% included some surface plots of 
% solutions at finer meshes; these
% were obtained via use of Sinc 
% interpolation formulas.
%
% Velocity fields at (x,y,0,T/2)
% 
% These are 
%
U1xy = U1(:,:,N+1,N+1);
U2xy = U2(:,:,N+1,N+1);
U3xy = U3(:,:,N+1,N+1);
%
% Velocity fields at (x,0,0,t) and
%
%  pressure at (x,0,0,t), (0,y,0,t), 
%              (0,0,z,t)
%
for ii=1:m
    for j=1:m
        U1xt(ii,j) = U1(ii,N+1,N+1,j);
        U2xt(ii,j) = U2(ii,N+1,N+1,j);
        U3xt(ii,j) = U3(ii,N+1,N+1,j);
        pxt(ii,j) = p(ii,N+1,N+1,j);
        pyt(ii,j) = p(N+1,ii,N+1,j);
        pzt(ii,j) = p(N+1,N+1,ii,j);
    end
end
%
% Finer mesh values and Sinc inter-
% polation
%
steps = (zs(m) - zs(1))/100;
stept = (zt(m) - zt(1))/100;
%
bb = zeros(100,m);
% cc = zeros(100,m);
dd = zeros(100,m);
for j=1:100
    xx = zs(1) + (j-0.5)*steps;
    tt = zt(1) + (j-0.5)*stept;
    bb(j,:) = basis1(xx,N,N,hs);
%    cc(j,:) = basis2(0,T,tt,N,N,ht);
    dd(j,:) = basis_poly2(0,T,tt,N,N,ht);
    ys(j) = xx;
    yt(j) =tt;
end
%
% Plots
%
% 1. u_1 at Sinc points
%
figure(1)
surf(zs,zs,U1xy)
pause
print -dps2 n_s2_u1_x_y_0_halfT.ps
%
% 2. u1 at finer mesh
%
% The reader should compare the 
% following figure(2) plot with 
% the Sinc point plot of figure(1).
%
u1_fine = bb*U1xy*bb.';
figure(2)
surf(ys,ys,u1_fine)
pause
print -dps2 n_s2_u1_fine_x_y_0_halfT.ps
%
% 3. u2 at Sinc points
% 
figure(3)
surf(zs,zs,U2xy)
pause
print -dps2 n_s2_u2_x_y_0_halfT.ps
%
% 4. u3 at Sinc points
% 
figure(4)
surf(zs,zs,U3xy)
pause
print -dps2 n_s2_u3_x_y_0_halfT.ps
%
% 5. u1(x,0,0,t) at Sinc points
%
figure(5)
surf(zt,zs,U1xt)
pause
print -dps2 n_s2_u1x_0_0_t.ps
%
% 6. u2(x,0,0,t) at Sinc points
%
figure(6)
surf(zt,zs,U2xt)
pause
print -dps2 n_s2_u2x_0_0_t.ps
%
% 7. u3(x,0,0,t) at Sinc points
%
figure(7)
surf(zt,zs,U3xt)
pause
print -dps2 n_s2_u3x_0_0_t.ps
%
% 8. u3(x,0,0,t) on finer mesh
%
% The reader should compare the
% following plot with the Sinc point 
% plot of figure(7)
%
u3xt_fine = bb*U3xt*dd.';
%
figure(8)
surf(yt,ys,u3xt_fine);
pause
print -dps2  n_s2_u3_fine_x_0_0_t.ps
%
% 9. Pressure at Sinc points (x,y,0,T/2)
%
Pxy = p(:,:,N+1,N+1);
%
figure(9) 
surf(zs,zs,Pxy)
pause
print -dps2 n_s2_p_x_y_0_halfT.ps
%
% 10. Pressure p(x,y,0,T/2) on 
% finer mesh.  Compare figs(9) and 10.
%
p_fine = bb*Pxy*bb.';
%
figure(10)
surf(ys,ys,p_fine)
pause
print -dps2 n_s2_p_fine_x_y_0_halfT.ps
%
% 11, 12,13. Pressures p(x,0,0,t), 
% p(0,y,0,t), and p(0,0,z,t)  at Sinc 
% points
%
figure(11) 
surf(zt,zs,pxt)
pause
print -dps2 pxt_x_0_0_t.ps
%
figure(12)
surf(zt,zs,pyt)
pause
print -dps2 pyt_0_y_0_t.ps
%
figure(13)
surf(zt,zs,pzt)
pause
print -dps2 pzt_0_0_z_t.ps
%
% 14. p(0,0,z,t) on a finer 
% mesh.  Compare figs(13) and (14).
%
pzt_fine = bb*pzt*dd.';
%
figure(14)
surf(yt,ys,pzt_fine)
pause
print -dps2 n_s2_pst_fine_0_0_z_t.ps


