%
% "navier_stokes_S0.m"
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
function S = navier_stokes_S0(u,v,w,t,e)
%
k = 1/sqrt(t*e);
k2 = k^2;
u2 = 1/u^2;
v2 = 1/v^2;
w2 = 1/w^2;
%
L2 = u2 + v2 + w2;
%
mu1 = sqrt(L2 - u2);
mu2 = sqrt(L2 - v2);
mu3 = sqrt(L2 - w2);
%
sq1 = sqrt(k2-mu1^2);
sq2 = sqrt(k2-mu2^2);
sq3 = sqrt(k2-mu3^2);
%
d1 = 1/(1/v + i/w);
cd1= 1/(1/v - i/w);
d2 = 1/(1/w + i/u);
cd2= 1/(1/w - i/u);
d3 = 1/(1/u + i/v);
cd3= 1/(1/u - i/v);
%
ze1 = i*cd1/d1;
ze2 = i*cd2/d2;
ze3 = i*cd3/d3;
z1p = d1*(-k + sq1);
z1m = d1*(-k - sq1);
z2p = d2*(-k + sq2);
z2m = d2*(-k - sq2);
z3p = d3*(-k + sq3);
z3m = d3*(-k - sq3);
%
F = 1/(4*pi);
%
H01 = -F/u/mu1*(Qfunc(ze1) - Qfunc(-ze1));
H02 = -F/u/mu2*(Qfunc(ze2) - Qfunc(-ze2));
H03 = -F/u/mu3*(Qfunc(ze3) - Qfunc(-ze3));
%
H1 = F/u/i/sq1*(Qfunc(z1p) - Qfunc(z1m));  
H2 = F/v/i/sq2*(Qfunc(z2p) - Qfunc(z2m));
H3 = F/w/i/sq3*(Qfunc(z3p) - Qfunc(z3m));
%
G3 = (k2-L2)*(1/8 - H1 - H2 - H3);
G03 = - 1/L2*(1/8 - H01 - H02 - H03);
DifH = H01 - H1;
S = t*(G03 - G3);
