%
% "ode_ivp_li1.m"
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% Here we solve the problem 
% 
% y' = f(x,y); y(0) = y_0 on (0,T),
%
% with f(x,y) = ode_f2(x,y) 
%             = - y/(1+x^2),
% If y0=1, then           
%           
% ode_f2_exact(x) = exp(arctan(x)).
%
% This problem is solved on 
% (0,T) for T = 0.5.  Again Picard 
% iteration converges for all (0,T)
% and while the exact Picard 
% converges to the exact answer, our 
% collocated Picard does so only if 
% T < 0.5.  It is suggested that 
% whenever the user does a Picard 
% she/h will do another run with 
% larger values of M and N, to see 
% if the two computations agree.  If not, 
% then she/he should use a smaller 
% interval.  
%
clear all
%
M=10;
N=10;
m = M+N+1;
h = 2.7/sqrt(N);
y0 = 1;
T = 0.5;
z = Sincpoints2(0,T,M,N,h);
A = Int2_p(0,T,M,N,h);
Er = 5;
%
for j=1:m
yy0(j) = y0;
end
yy = yy0.';
% 
while(Er > .00001)
     y = yy;
     for k=1:m
        w(k) =  ode_f2(z(k),y(k));
     end   
%
     Z = w.';
% 
     yy = yy0.'+A*Z;
     Er = norm(y - yy);
end
%
for j=1:m
   ex2(j)= ode_f2_exact(z(j));
end
%
TT = ex2.';
plot(z,TT,'o',z,yy)
%
