%
% `pois_R_3.m'
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% This is a Sinc convolution solution to the problem 
%
% \nabla u = - (6 - 4 r^2) exp(-r^2)
%
% via Sinc methods -- see Sec. 4.1.5 of this manual.
%
% The exact sololution is u(r) = exp(-r^2).
%
% Parameters
%
clear all
%
M1 = 14;
N1 = M1;
m1 = M1+N1+1;
h1 = 1.1/sqrt(N1);
M2 = M1;
N2 = M1;
m2 = m1;
h2 = h1;
M3 = M1;
N3 = M1;
m3 = m1;
h3 = h1;
%
% Sinc points
%
c = -M1*h1-h1;
for j=1:m1
   z1(j) = j*h1+c;
end
%
z2 = z1;
z3 = z1;
%
% Evaluate the function g
%
for j=1:m1
   for k=1:m2
      for l=1:m3
         r2 = z1(j)^2+z2(k)^2+z3(l)^2;
         g(j,k,l) = - (4*r2-6)*exp(-r2);
      end
   end
end
%
clear r2
%
% Indefinite integration matrices.  These 
% were also computed in previous examples.
%
A1 = h1*Im(m1);
B1 = A1.';
A2 = h2*Im(m2);
B2 = A2.';
A3 = h3*Im(m3);
B3 = A3.';
%
% Eigenvalues and eigenvectors
%
[X1 S1] = eig(A1);
[Y1 T] = eig(B1); 
s1 = diag(S1);
%
[X2 S2] = eig(A2);
[Y2 T] = eig(B2); 
s2 = diag(S2);
%
[X3 S3] = eig(A3);
[Y3 T] = eig(B3); 
s3 = diag(S3);
%
%
% Evaluate the 3d Laplace transform, 
% as given in Lemma 5.6.3 of [SincTut].
%
for j=1:m1
   for k=1:m2
      for l=1:m3
         G(j,k,l) = lap_tr_poi3(s1(j),s2(k),s3(l));
      end
   end
end
%
%
% Now, solve the PDE Lap U = - g
% via repeated (8 times) use of the convolution 
% algorithm, Algorithm 5.5.2 of [SincTut].
%
U = zeros(m1,m2,m3);
%
for n=1:8
   if n == 1 
     A=X1; B=X2; C=X3;
   elseif n == 2
     A=X1; B=X2; C=Y3;
   elseif n == 3
     A=X1; B=Y2; C=X3;
   elseif n == 4 
     A=X1; B=Y2; C=Y3;
   elseif n == 5
     A=Y1; B=X2; C=X3;
   elseif n == 6 
     A=Y1; B=X2; C=Y3;
   elseif n == 7
     A=Y1; B=Y2; C=X3;
   else
     A=Y1; B=Y2; C=Y3;
   end
   U  = U + conv3_vec(A,m1,B,m2,C,m3,G,g);
end
%
% We now do some surface, and cross section plots.
%
UU = real(U);
clear U
V1 = UU(:,:,M3+1);
surf(z1,z2,V1);
title('Solution U in Plane $z = 0$')
print -dps2 pois_3_d_appr.ps
pause
for j=1:m1
   for l=1:m3
      V2(j,l) = UU(j,M2+1,l);
   end
end
surf(z2,z3,V2);
pause
for k=1:m2
   for l=1:m3
      V3(k,l) = UU(M1+1,k,l);
   end
end
surf(z2,z3,V3);
pause
for j = 1:m1
   exact(j) = exp(-z1(j)^2);
end
plot(z1,UU(:,M2+1,M3+1),z1,exact,'o')
title('Error plot in Planes $y = z = 0$')
print -dps2 gauss_err.ps
pause
plot(z3,V2(M1+1,:))
pause
plot(z2,V3(:,M3+5))
pause
clear V1
clear V2
clear V3

