%
% Program: `terryT4.m' 
%
% This program is copyrighted by SINC, LLC
% an belongs to SINC, LLC
% 
% Sinc Solution of 
%
% F(x,t) = F_0(x) + U(x,t) - V(x,t) 
%
% over 
%
% (0,infinity) x {(0,T), (T,2T), (2T,3T) and (3T,4T)} 
%
% Here, 
%
%   U(x,t) = int_0^t int_x^infinity G(x,u) b(u) "times" 
%               "times" F_x(u,tau) du dtau 
%
%   V(x,t) = int_0^t mu(tau) int_0^x F_x(v,tau) "times" 
%     "times" int_{x-v}^infinity w^{-1} a(v,w) "times" 
%      "times" F_x(w,tau) dw dv dtau
%
clear all
M1 = 20;
N1 = 15;
m1 = M1 + N1 + 1;
M2 = 15;
N2 = 15;
m2 = M2 + N2 + 1;
ht = 0.7/sqrt(N2);
hx = 3.2/sqrt(N1);
T = 0.25;
%
format long e
%
% Sinc Points, Weights, and Matrices
%
x = Sincpoints4(M1,N1,hx);
wx = weights4(M1,N1,hx);
%
tau = Sincpoints2(0,T,M2,N2,ht);
wt = weights2(0,T,M2,N2,ht);
%
% First, some definitions of functions and 
%
% some diagonal matrices.
%
for j=1:m1
% the initial F_0
F0(j) = 1 - exp(- x(j)/0.05);
% 
b(j) = 4;
for k=1:m1
% The next two are values of G and a 
 G(j,k) = (x(j)/x(k))^0.5/(20/3);
 a(j,k) = x(j) + x(k);
end
end
%
% The diagonals of full matrices
%
for j=1:m1
for k=1:m2
%
% The initial value of F
%
FF0(j,k) = F0(j);
end
end
%
% The approximation of "mu"
%
Dmu = eye(m2);
%
for k=1:m2
Dmu(k,k) = .05;
end
%
% The matrices
%
% First, the integral matrices
%
Axp = Int4_p(M1,N1,hx);
Axm = Int4_m(M1,N1,hx);
%
% Next, The "x" derivative matrix, as the inverse of 
% the above Axp matrix: 
%
Axd = inv(Axp);
%
% Integration over (0,t)
%
Atp = Int2_p(0,T,M2,N2,ht);
% Now diagonalize the matrix Axp, for 
% Sinc convolution
%
[Y S] = eig(Axp);
s = diag(S);
% This is a row vector of the diagonal 
% entries of S
Yi = inv(Y);
%
% Beginning of "time-strip", ll - loop, to compute 
% the solution F on the time interval I_l = ((ll-1)T,(ll)T)
%
%
for ll=1:4
%
% A bit more initialization for I_l 
    for j=1:m2
        t(j) = (ll-1)*T+tau(j); % Sinc points on  I_l 
        Dmu(j,j) = .05;
    end
%
%
% Start of iterative "contraction"  loop
%
% 
F = FF0;  % FF0 = F(x,t)|_{t=0}
nn = 1; 
% The value of the "least squares" 
% convergence norm, is here set to 1
while(nn > 0.0001)
%
% First, compute the U = U(x,t) - Part
%
% F_x denoted by Fx
%
Fx = Axd * F;
%
% Next, G(x,u) b(u) F_x(u,tau)
%
for j=1:m1
    for k=1:m2
        for l=1:m1
            GbFx(j,k,l) = G(j,l) * b(l) * Fx(l,k);
        end
    end
end
%
% P2(x,tau) = int_x^infinity G(x,u) b(u) F_x(u,tau) du
%
for j=1:m1
    for k=1:m2
        P2(j,k) = 0;
        for l=1:m1
            P2(j,k) = P2(j,k) + Axm(j,l) * GbFx(j,k,l);
        end
    end
end
%
clear GbFx
%
% We now get U(x,t) at Sinc pts.:
% U(x,t) = int_0^t P2(x,tau) dtau
%
U = P2 * Atp';
% 
clear P2
%
%
%  V(x,t) is next:
%
%
for j=1:m1
    for k=1:m2
        for l=1:m1
            awFx(j,l,k) = a(j,l)/x(l)*Fx(l,k); 
        % awFx(v,w,tau) = a(v,w)/w*F_x(w,tau)
        end
    end
end
%
% Set P2(y,v,tau) = int_y^infinity awFx(v,w,tau) dw
% 
for n=1:m1
    for j=1:m1
        for k=1:m2
            sum=0;
            for l=1:m1
                sum = sum + Axm(n,l)*awFx(j,l,k);
            end
            P2(n,j,k) = sum;
        end
    end
end
%
clear awFx
%
% Want: P4(x,tau) = int_0^x P2(x-v,v,tau) Fx(v,tau) dv
%
% Method:  Consider P(x) = int_0^x G(x-v,v) dv.
% 
% To get P = (P_1, ..., P_m)^T , need 
%
% H(s,v) = int_0^infinity G(y,v) exp(-y/s) dy.
% 
% Then, with Axp = Y S Y^{-1}, where Y^{-1} = [y^{jk}],
%
% and Q = (Q_1, ..., Q_m)^T = Y^{-1}P, we have 
%
% Q_j = sum_l y^{jl} H(s_j,v_l), from which we get
%
% P = Y Q.
%
% First, need the 
% Laplace transform of P2(y,v,t) wrt y:
%
%  P3(s,v,tau) = sum_n wx(n) P2(n,j,k) exp(-x(n)/s)
%
for j=1:m1
    for k=1:m2
        for l=1:m1
            sum = 0;
            for n=1:m1
                sum = sum + wx(n)* P2(n,j,k)*exp(-x(n)/s(l));
            end
            P3(l,j,k) = sum;
        end
    end
end
%
clear P2
%
% Set P4(x,tau) = int_0^x P2(x-v,v,tau) Fx(v,tau) dv
% Split into two parts
% First part
%
for j=1:m1
    for k=1:m2
        sum = 0;
        for l=1:m1
            sum = sum+Yi(j,l)*P3(j,l,k)*Fx(l,k);
        end
        PP4(j,k) = sum;
    end
end
%
clear P3
% Second part
P4 = Y * PP4;
% 
clear PP4 
%
% Finally, V
%
V = P4 * Dmu * Atp.';
%
FF = FF0 + U - V;
nn0 = norm(F - FF);
W = real(FF);
% Either F = FF; or the next line
F = W;
nn = nn0
%
% End of "contraction iteration" loop
%
end 
%
x=x';
t=t';
surf(t,x,W)
pause
%
% Printing a postscript file of this surface
%
if(ll<2) 
    print -dps2 terrT.ps
pause
%
% Viewing a cross Section, U(x,t) for 
% x(1) <= x <= x(m) and t = t1 = 0.6
%
t1 = 0.6*T;
uu = log(t1/(1-t1));
t_bas_vec = basis(uu,M2,N2,ht);
vv = W*t_bas_vec';
hh = (x(m1) - x(1))/400;
for j=1:400
xx(j) = x(1) + (j-0.5)*hh;
phy = log(sinh(xx(j)));
x_bas = basis(phy,M1,N1,hx);
     F_coord(j) = x_bas*vv;
end
semilogx(xx,F_coord)
pause
semilogx(x,W);
pause
elseif(ll>1) & (ll<3)
    print -dps2 terr2T.ps
pause
%
% Viewing a cross Section, U(x,t) for 
% x(1) <= x <= x(m) and t = 1.6
%
t1 = 1.6*T-T;
uu = log(t1/(T-t1));
t_bas_vec = basis(uu,M2,N2,ht);
vv = W*t_bas_vec';
hh = (x(m1) - x(1))/400;
for j=1:400
xx(j) = x(1) + (j-0.5)*hh;
phy = log(sinh(xx(j)));
x_bas = basis(phy,M1,N1,hx);
     F_coord(j) = x_bas*vv;
 end
 semilogx(xx,F_coord)
pause
semilogx(x,W);
pause
elseif(ll>2) & (ll<4)
    print -dps2 terr3T.ps
pause
%
% Viewing a cross Section, U(x,t) for 
% x(1) <= x <= x(m) and t = t1 = 2.6
%
t1 = 2.6*T - 2*T;
uu = log(t1/(T-t1));
t_bas_vec = basis(uu,M2,N2,ht);
vv = W*t_bas_vec';
hh = (x(m1) - x(1))/400;
for j=1:400
xx(j) = x(1) + (j-0.5)*hh;
phy = log(sinh(xx(j)));
x_bas = basis(phy,M1,N1,hx);
     F_coord(j) = x_bas*vv;
 end
 semilogx(xx,F_coord)
pause
semilogx(x,W);
pause
elseif(ll>3)
    print -dps2 terr4T.ps
pause
%
% Viewing a cross Section, U(x,t) for 
% x(1) <= x <= x(m) and t = t1 = 3.6
%
t1 = 3.6*T-3*T;
uu = log(t1/(T-t1));
t_bas_vec = basis(uu,M2,N2,ht);
vv = W*t_bas_vec';
hh = (x(m1) - x(1))/400;
for j=1:400
xx(j) = x(1) + (j-0.5)*hh;
phy = log(sinh(xx(j)));
x_bas = basis(phy,M1,N1,hx);
     F_coord(j) = x_bas*vv;
 end
 semilogx(xx,F_coord)
pause
semilogx(x,W);
end
%
% Initialization for next I_l
%
for j=1:m1
    for k=1:m2
        FF0(j,k) = W(j,m2);
    end
end
%
% End of "time-strip" loop
%
end
    
%

        





