%
% Program: `terry_br.m' 
%
% This program is copyrighted by SINC, LLC
% an belongs to SINC, LLC
% 
% Sinc Solution of 
%
% (***) F(x,t) = F_0(x) + U(x,t) - V(x,t) 
%
% over 
%
% (0,infinity) x {(0,infinity) via Sinc combined 
% with Broyden's method.
%
% Here, 
%
%   U(x,t) = int_0^t int_x^infinity G(x,u) b(u) "times" 
%               "times" F_x(u,tau) du d\tau
%
%   V(x,t) = int_0^t mu(tau) int_0^x F_x(v,tau) "times" 
%     "times" int_{x-v}^infinity w^{-1} a(v,w) "times" 
%      "times" F_x(w,tau) dw dv dtau
%
clear all
format long e
M1 = 8;
N1 = 10;
m1 = M1 + N1 + 1;
M2 = 10;
N2 = 8;
m2 = M2 + N2 + 1;
ht = 0.6/sqrt(N2);
hx = 1.4/sqrt(N1);
m3 = m1*m2;
%
% Sinc Points, Weights, and Matrices
%
x = Sincpoints4(M1,N1,hx);
wx = weights4(M1,N1,hx);
%
t = Sincpoints4(M2,N2,ht);
wt = weights4(M2,N2,ht);
%
% First, some definitions of functions and 
%
% some diagonal matrices.
%
for j=1:m1
%
% the initial F_0
%
   F0(j) = 1 - (1 + x(j)) * exp(- x(j));
   b(j) = 1/(1+x(j)^2); % b(x) = 1/(1+x^2)
       for k=1:m1
%
% The next two are values of G and a
% 
           G(j,k) = x(j)/(x(j)^2 + x(k)^2+1);
           a(j,k) = x(j)/(x(j)^2 + x(k)^2+1);
       end
end
%
for j=1:m1
    for k=1:m2
%
% The initial value of F
%
        FF0(j,k) = F0(j);
    end
end
%
% The approximation of "mu" as a matrix
%
Dmu = eye(m2);
%
for k=1:m2
    Dmu(k,k) = 1/(1+t(k)^2);
end
%
% Indefinite integration matrices
%
% With respect to x:
Axp = Int4_p(M1,N1,hx);
Axm = Int4_m(M1,N1,hx);
%
% With respect to t:
Atp = Int4_p(M2,N2,ht);
% The derivative matrices
%
% The "x" derivative matrix on (0,infinity)
%
% Axd = D44_1(M1,N1,hx);
% Axd = inv(Axp);
% Axd = inv(Axm); % This one gives bad results
 Axd = 0.5*(inv(Axp) + inv(Axm));
%
% Now diagonalize the matrix Axp, for 
% Sinc convolution
%
[Y S] = eig(Axp);
s = diag(S);
clear S;
%
% This is a vector of the diagonal entries of S
%
% Start of the Broyden Routine
%
% The initial value of F
%
for j=1:m1
    for k=1:m2
        FF0(j,k) = F0(j);
    end
end
%
% First, the initial Broyden inverse of the Jacobian:
%
BBB = eye(m3);
%
% We now call the "F-evaluation" routine.  This is just the 
% Sinc-approximate collcocation of the right hand side
% of (***) above at the Sinc points.
% 
% Initially, we use the above computed FF0
%
F = FF0;
%
F00 = GGG(m1,m2,x,wx,t,wt,Y,s,b,G,a,FF0,F,Dmu,Axd,Axp,Axm,Atp);
G00 = F - F00;
%
% Changing matrix to vector:
%
for j=1:m2
    for  i=1:m1
        l = i + (j-1)*m1;
        g00(l) = G00(i,j);
        f0(l) = F(i,j);
    end
end
%
    nn = 1;
%
% Quasi-Newton onvergence norm is here set to 1
%
while(nn > 0.001)
    %
    ss = -BBB*g00.';
    f1 = f0 + ss.';
    %
    for j=1:m2
        for  i=1:m1
            l = i + (j-1)*m2;
            F(i,j) = f1(l);
        end
    end
    %
    F01 =  GGG(m1,m2,x,wx,t,wt,Y,s,b,G,a,FF0,F,Dmu,Axd,Axp,Axm,Atp);
    G01 = F - F01;
    YY = G01-G00;
    %
    for j=1:m2
        for  i=1:m1
            l = i + (j-1)*m1;
            y(l) = YY(i,j);
            g00(l) = G01(i,j);
        end
    end
    %
    nn = norm(ss)
    %
    BBB = BBB + (ss - BBB*y.')*y/(y*y.');
    F00 = F01;
    f0 = f1;
    G00 = G01;
end
%
W = F01;
%
x=x';
t=t';
surf(t,x,W)
% pause
%



        





