%
% Program: `terry_broy.m' 
%
%
% This program is copyrighted by SINC, LLC
% an belongs to SINC, LLC
% 
% Sinc Solution of 
%
% (1) f(x,t) = ff0(x) + U(x,t) - V(x,t) 
%
% over (x,t) in (0,infinity) x (0,T1), by 
% 
% Broyden's method.
%
% Here, 
%
% (2)  U(x,t) = int_0^t int_x^infinity G(x,u) b(u) "times" 
%               "times" F_x(u,tau) du dtau 
%
% (3)  V(x,t) = int_0^t mu(tau) int_0^x F_x(v,tau) "times" 
%     "times" int_{x-v}^infinity w^{-1} a(v,w) "times" 
%      "times" F_x(w,tau) dw dv dtau
%
clear all
M1 = 12;
N1 = 8;
m1 = M1 + N1 + 1;
M2 = 8;
N2 = 8;
m2 = M2 + N2 + 1;
ht = 0.5/sqrt(N2);
hx = 1/sqrt(N1);
 T1 = 0.25;
%
format long e
%
% Sinc Points, Weights, and Matrices
%
x = Sincpoints4(M1,N1,hx);
% x = Sincpoints3(M1,N1,hx);
 wx = weights4(M1,N1,hx);
% wx = weights3(M1,N1,hx);
%
% t = Sincpoints4(M2,N2,ht);
% t = Sincpoints3(M2,N2,ht);
t = Sincpoints2(0,T1,M2,N2,ht);
wt = weights2(0,T1,M2,N2,ht);
% wt = weights4(M2,N2,ht);
% wt = weights3(M2,N2,ht);
%
% First, some definitions of functions and 
% some diagonal matrices.
%
for j=1:m1
% the initial condition 
   vecf0(j) = 1 - exp(- x(j)/0.1);
   % vecf0(j) = 1 - (1 + x(j)) * exp(- x(j));
% 
    b(j) = 1/(1+x(j)^2);
    % b(j) = 1;
    for k=1:m1
% The next two are values of G and a 
        % G(j,k) = (x(j)/(1+x(k)))^(0.3);
        G(j,k) = 1;
        % a(j,k) = (x(j) + x(k))/(x(j)^2 + x(k)^2+1);
        a(j,k) = 1;
    end
end
%
% The approximation of "mu"
%
Dmu = eye(m2);
%
for k=1:m2
    % Dmu(k,k) = .02*exp(-t(k));
    Dmu(k,k) = 1/(1+t(k)^2);
end
%
% Indefinite integration matrices
%
 Axp = Int4_p(M1,N1,hx);
% Axp = Int3_p(M1,N1,hx);
Axm = Int4_m(M1,N1,hx);
% Axm = Int3_m(M1,N1,hx);
%
% Next, The "x" derivative matrix, as the inverse of 
% the above Axp matrix: 
%
% Axd = inv(Axp);
  Axd = D44_1(M1,N1,hx);
% Axd = inv(Axm);
%
% Integration over (0,infinity)
%
Atp = Int4_p(M2,N2,ht);
% Atp = Int3_p(M2,N2,ht);
% Now diagonalize the matrix Axp, for 
% Sinc convolution
%
[Y S] = eig(Axp);
s = diag(S);
% This is a row vector of the diagonal 
% entries of S
Yi = inv(Y);
%
% Start of the Broyden routine
%
% The initial value of F
%
for j=1:m1
    for k=1:m2
        ff0(j,k) = vecf0(j);
    end
end
%
m3 = m1*m2;
%
% First, the initial Broyden inverse of the Jacobian:
%
cc = 0.5;
BBB = cc*eye(m3);
AAA = 1/cc*eye(m3);
%
% We now call the "f-evaluation" routine.  This is just the 
% Sinc-approximate collcocation of the right hand side
% of (***) above at the Sinc points.
% Initially, we use the above computed ff0
%
f0 = ff0;
fv0 = reshape(f0,m3,1);
%
F0 = ter_GGG(m1,m2,x,wx,t,wt,Y,s,b,G,a,ff0,f0,Dmu,Axd,Axp,Axm,Atp);
Fv0 = reshape(F0,m3,1);
fv1 = fv0 - BBB*Fv0;
f1 = reshape(fv1,m1,m2);
%
% Quasi-Newton onvergence norm is here set to 1
%
nn = 1;
%
% Beginning of quasi-Newton loop
%
while(nn > 0.0001)
%
  F1 = ter_GGG(m1,m2,x,wx,t,wt,Y,s,b,G,a,ff0,f1,Dmu,Axd,Axp,Axm,Atp);
  Fv1 = reshape(F1,m3,1);
  v = fv1-fv0;
  u = (Fv1 - Fv0 - AAA*v)/(1 + v.'*v);
  % u = (Fv1 - Fv0 - AAA*v);
  % u = (Fv1 - Fv0 - AAA*v)/(v.'*v);
  ss = u*v.';
  AAA = AAA + ss;
  BBB = BBB - BBB*ss*BBB/(1 + v.'*BBB*u);
  ss =  BBB*Fv1;
  fv2 = fv1 - ss ;
  fv0 = fv1;
  fv1 = fv2;
  f1 = reshape(fv1,m1,m2);
  % surf(t,x,f1);
  % pause
  nn = norm(ss)
%
end
% 
surf(t,x,f1)
