%
% Program:  `terryinf.m'
%
% This program is copyrighted by SINC, LLC
% an belongs to SINC, LLC
% 
% F(x,t) = F_0(x) + U(x,t) - V(x,t) 
%
%   U(x,t) = int_0^t int_x^infinity G(x,u) b(u) "times" 
%               "times" F_x(u,tau) du dtau 
%
%   V(x,t) = int_0^t mu(tau) int_0^x F_x(v,tau) "times" 
%     "times" int_{x-v}^infinity w^{-1} a(v,w) "times" 
%      "times" F_x(w,tau) dw dv dtau
%
% A solution over (x,t) in (0,infty) x (0,infty) 
% is computed
%
clear all
%
% Initial value inputs
%
M1 = 12;
N1 = 12;
m1 = M1 + N1 + 1;
M2 = 12;
N2 = 12;
m2 = M2 + N2 + 1;
ht = 1/sqrt(N2);
hx = 2.2/sqrt(N1);
%
format long e
%
% Sinc Points, Weights, and Matrices
%
x = Sincpoints4(M1,N1,hx);
wx = weights4(M1,N1,hx);
%
t = Sincpoints4(M2,N2,ht);
wt = weights4(M2,N2,ht);
%
% First, some definitions of functions and 
%
% some diagonal matrices.
%
for j=1:m1
% the initial F_0
F0(j) = 1 - (1 + x(j)) * exp(- x(j)); 
% F0(j) = 1 - (1 + x(j)^2) * exp(- x(j)^2);
b(j) = 1/(1+x(j)^2); % b(x) = 1/(1+x^2)
for k=1:m1
% The next two are values of G and a 
 G(j,k) = x(j)/(x(j)^2 + x(k)^2+1);
 a(j,k) = x(j)/(x(j)^2 + x(k)^2+1);
end
end
%
% The diagonals of full matrices
%
for j=1:m1
for k=1:m2
%
% The initial value of F
%
FF0(j,k) = F0(j);
end
end
%
% The approximation of "mu"
%
Dmu = eye(m2);
%
for k=1:m2
Dmu(k,k) = 1/(1+t(k)^2);
end
%
% The derivative matrices
%
% The "x" derivative matrix on (0,infinity)
%
Axd = D44_1(M1,N1,hx);
%
% Integration over (0,x) and (x,infinity)
%
Axp = Int4_p(M1,N1,hx);
Axm = Int4_m(M1,N1,hx);
%
% Integration over (0,t)
%
Atp = Int4_p(M2,N2,ht);
% Now diagonalize the matrix Axp, for 
% Sinc convolution
%
[Y S] = eig(Axp); 
s = diag(S); 
% This is a vector of the diagonal entries of S
%
% Start of iterative "contraction"  loop
%
% 
F = FF0;  % FF0 = F(x,t)|_{t=0}
nn = 1; % The value of the "least squares" 
% convergence norm, is here set to 1
while(nn > 0.001)
%
% First, compute the U = U(x,t) - Part
%
% F_x denoted by Fx
%
Fx = Axd * F;
%
% Next, G(x,u) b(u) F_x(u,tau)
%
for j=1:m1
for k=1:m2
    for l=1:m1
	GbFx(j,k,l) = G(j,l) * b(l) * Fx(l,k);
    end
end
end
%
% P2(x,tau) = int_x^infinity G(x,u) b(u) F_x(u,tau) du
%
for j=1:m1
for k=1:m2
    P2(j,k) = 0;
    for l=1:m1
	P2(j,k) = P2(j,k) + Axm(j,l) * GbFx(j,k,l);
    end
end
end
%
clear GbFx
%
% We now get U(x,t) at Sinc pts.:
% U(x,t) = int_0^t P2(x,tau) dtau
%
U = P2 * Atp';
% 
clear P2
%
%
%  V(x,t) is next:
%
%
for j=1:m1
for k=1:m2
    for l=1:m1
	awFx(j,l,k) = a(j,l)/x(l)*Fx(l,k); 
	% awFx(v,w,tau) = a(v,w)/w*F_x(w,tau)
    end
end
end
%
% Set P2(y,v,tau) = int_y^infinity awFx(v,w,tau) dw
% 
for n=1:m1
for j=1:m1
    for k=1:m2
	sum=0;
	for l=1:m1
	    sum = sum + Axm(n,l)*awFx(j,l,k);
	end
	P2(n,j,k) = sum;
    end
end
end
%
clear awFx
%
% Want: P4(x,tau) = int_0^x P2(x-v,v,tau) Fx(v,tau) dv
%
% Method:  Consider P(x) = int_0^x G(x-v,v) dv.
%
% See Sec. 4.3.11 of [SincTut], and Sec. 5.8.1 of 
%
% this manual
% 
% To get P = (P_1, ..., P_m)^T , need 
%
% H(s,v) = int_0^infinity G(y,v) exp(-y/s) dy.
% 
% We then use the Sinc-Pack routine convnl1_t.m 
%
% to get P.
%
% First, need the 
%
% Laplace transform of P2(y,v,t) wrt y, via Sinc 
%
% quadrature.
%
%  P3(s,v,tau) = sum_n wx(n) P2(n,j,k) exp(-x(n)/s)
%
for k=1:m2
   for j=1:m1
      for l=1:m1
	  sum = 0;
	  for n=1:m1
	    sum = sum + wx(n)* P2(n,j,k)*exp(-x(n)/s(l));
	  end
	  P3(l,j,k) = sum*Fx(j,k);
      end
   end
   PP4 = P3(:,:,k);
   Q4 = convnl1_t(Y,m1,PP4);
   for j=1:m1
      P4(j,k) = Q4(j);
   end
end
%
clear P2
clear P3
clear PP3
clear Q4
%
% Finally, V
%
V = P4 * Dmu * Atp.';
%
FF = FF0 + U - V;
nn0 = norm(F - FF);
W = real(FF);
% Either F = FF; or the next line
F = W;
nn = nn0
%
% End of "contraction iteration" loop
%
end 
%
x=x';
t=t';
surf(t,x,W)
pause
%
% Viewing a cross Section, U(x,t) for 
% x(1) <= x <= x(m) and t = t1 = 1.6
%
t1 = 1.6;
uu = log(sinh(t1));
t_bas_vec = basis(uu,M2,N2,ht); 
vv = W*t_bas_vec';
hh = (x(m1) - x(1))/400;
for j=1:400
    xx(j) = x(1) + (j-0.5)*hh;
    phy = log(sinh(xx(j)));
    x_bas = basis(phy,N1,N2,hx);
    F_coord(j) = x_bas*vv;
end
 plot(xx,F_coord);


