%
% `wavelet_even_half_Int_p.m'
%
function [A] = wavelet_even_half_Int_p(N,a,b)
%
% This program is copyrighted by SINC, LLC
% and belongs to SINC, LLC
%
% Wavelet_even_half_Int_p(N,a,b) computes the 
% matrix A = [A_{jk}] of order M = 2N, where,
% with P = b - a, and for k in [1,M], 
% A_{jk} = E_e(N,P,x_j-x_k) - E_e(N,P,a-x_k)
% and with x_k = a + (k-1/2)P/(2N), k=1, ... 2N.
% See (1.4.105).
%
% NOTES:
%
% 1.  A_{jk} is the coeff, of f(a + (k-1/2)h) when 
% integrating the right hand side of (1.4.18) from 
% a to x_j = a + (j-1/2)h.
% 
% 2. Setting f_j = f(a + (j-1/2)h), j = 1, 2, ... 2N, 
% f = (f_1, ..., f_{2N})^T, and F = Af,
% we can again use the formula on the right
% hand side of (1.4.18) to get an approximation of 
% F(x) = int_a^x f_h(t) dt for any x in [a, b].  
% The program `wavelet_basis_even_int.m' may be used 
% for this purpose, to yield an interpolation that is 
% exact at the interpolation points x_j.
%
P = b - a;
M = 2*N;
h = P/M;
%
for k=1:M
    x(k) = a + (k-1/2)*h;
end
%
for k=1:M;
    bb = waveletint_e(N,P,a - x(k));
    for j=1:M;
        A(j,k) = waveletint_e(N,P,x(j)-x(k)) - bb;
    end
end
%
